#!/usr/bin/env python
"""
U2FsdGVkX19cV2plTMSqbpdFaJuEChbUka1NX+X4qeSMMFuTquH1Wd+vL6e2Hf4H
CAvb+R3+MDOCjdOyhgS3DNleCDqZRRJorq7lehTQzbA/pY9TMQBMyUnm63wUOjd/
EKVAggOM+jODpiFn0+FN4le8LUoOErVSZSR7L4gka/7L69wT82r90gq12H1mFNns
judHXORdh3Iud6J8UQHqSZUSFRfnd1IetIIySIIVxxCPE0DY2CS7Y/TjmEiHduzm
l4TiHrn9MlQOJY21PIjswrB9EIS/fsQsNV09BIBh6cJTu2jPULRjOpVGw+liyvkJ
I6HynL0O7XEVC61l2wiQJA==
"""

import construct

from base_msg import PSBP, SOURCE_ADDRESS
from utils import fmt_repr, containerize, exclude_fields

PSBP_MSG_VOLTAGE_REQ = 0x08


class MsgVoltageReq(PSBP):
    """
    voltage request
    """

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgVoltageReq,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)
            super.from_binary(psbp.payload)
        else:
            super(MsgVoltageReq, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_VOLTAGE_REQ

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = super(MsgVoltageReq, self)._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))


PSBP_MSG_VOLTAGE_ACK = 0x08


class MsgVoltageAck(PSBP):
    """
    voltage ack
    """
    _parser = construct.Struct(
        'voltage' / construct.Int16ub)
    __slots__ = [
        'voltage',
    ]

    def __init__(self, psbp=None, **kwargs):
        if psbp:
            super(MsgVoltageAck,
                  self).__init__(psbp.receiver_address, psbp.source_address, psbp.data_length, psbp.flag,
                                 psbp.command, psbp.payload, psbp.crc)

            self.from_binary(psbp.payload)
        else:
            super(MsgVoltageAck, self).__init__()
            self.receiver_address = kwargs.pop('receiver_address')
            self.source_address = kwargs.pop('source_address', SOURCE_ADDRESS)
            self.flag = kwargs.pop('flag')
            self.command = PSBP_MSG_VOLTAGE_ACK
            self.voltage = kwargs.pop('voltage')

    def __repr__(self):
        return fmt_repr(self)

    def from_binary(self, d):
        """Given a binary payload d, update the appropriate payload fields of
        the message.

        """
        p = MsgVoltageAck._parser.parse(d)
        for n in self.__class__.__slots__:
            setattr(self, n, getattr(p, n))

    def to_binary(self):
        """Produce a framed/packed PSBP message.

        """
        c = containerize(exclude_fields(self))
        self.payload = MsgVoltageAck._parser.build(c)
        return self.pack()


msg_classes = {
    0x08: MsgVoltageAck,
}
